@@ -25,6 +25,7 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <gdk/gdkkeysyms.h>
 
 #include <fcntl.h>
 #include <stdio.h>
@@ -39,7 +40,6 @@
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/socket.h>
 
 #ifdef linux
 # include <linux/cdrom.h>
@@ -48,18 +48,12 @@
 # error TCD currently only builds under Linux systems.
 #endif
 
-#include "cddb.h"
 #include "gtracked.h"
-#include "gabout.h"
-#include "gcddb.h"
-#include "properties.h"
-#include "tooltips.h"
+#include "prefs.h"
 #include "led.h"
-
-/* callback commands */
-enum { PLAY=0, PAUSE, STOP, EJECT,
-       FF, RW, NEXT_T, PREV_T,
-       CDDB, TRACKLIST, GOTO, QUIT, ABOUT, PROPS };
+#include "callbacks.h"
+#include "gtcd_public.h"
+#include "keybindings.h"
 
 /* time display types */
 enum { TIME_FIRST=-1, TRACK_E, TRACK_R, DISC_E, DISC_R, TIME_LAST };
@@ -72,216 +66,214 @@ char *display_types[] =
     "dsc-r"
 };
 
-char *play_methods[] =
+char *play_types[] = 
 {
-    "repeat_cd.xpm",
-    "repeat_track.xpm",
-    "repeat_normal.xpm",
-    "repeat_random.xpm" // Not yet implemented
+    "loop-cd",
+    "loop-t",
+    "normal",
+    "random"
 };
-GtkWidget *play_method_pixmap[4];
 
-#define Connect( x,y ) gtk_signal_connect (GTK_OBJECT (x), "clicked", \
-					   GTK_SIGNAL_FUNC (callback), (gpointer*)y);
-
-/* Regular globals */
+/* globals */
 cd_struct cd;
 int titlelabel_f = 0;
 int gotoi;
 
-/* Gtk Globals */
 GtkWidget *row, *upper_box;
-GtkWidget *button_box, *row1, *row2, *row3;
-GtkWidget *trackeditor;
+GtkWidget *button_box;
 GtkWidget *tracktime_label, *trackcur_label;
 GtkWidget *cdtime_label, *changer_box, *playbutton;
-GtkWidget *status_table, *status_area, *sep;
-GtkWidget *volume, *window, *aboutbutton, *propsbutton;
+GtkWidget *status_table, *status_area;
+GtkWidget *volume, *window;
 GtkWidget *gotomenu = NULL, *gotobutton, *main_box;
 GdkPixmap *status_db = NULL;
-GtkWidget **changer_buttons, *cddbbutton;
+GtkWidget **changer_buttons;
 GtkObject *vol;
 GdkColormap *colormap;
 GdkFont *sfont, *tfont;
 GdkColor track_color, darkgrey, timecolor, trackcolor;
+GdkGC *gc;
+GtkAccelGroup *accel;
 
 GtkTooltips *tooltips;
 
 int timeonly = FALSE, status_height, status_width, playid=-1;
 int configured = FALSE, old_status=-1, max,tfont_height;
-unsigned int cur_goto_id;
-tcd_properties props;
+unsigned int cur_goto_id, release_t=0, roll_t=0;
+tcd_prefs prefs;
 
-/* Prototypes */
-void draw_status( void );
-void delete_event (GtkWidget *widget, gpointer *data);
-void create_warn( char *, char * );
-void make_gotomenu();
+/* prototypes */
 void status_changed(void);
+gint slow_timer(gpointer *data);
+gint roll_timer(gpointer *data);
+gint release_timer(gpointer *data);
+int skip_cb(GtkWidget *widget, GdkEvent *event, gpointer *data);
+gint changer_callback(GtkWidget *widget, gpointer *data);
+GtkWidget* make_changer_buttons(void);
+GtkWidget* make_small_buttons(void);
+GtkWidget* create_buttons(void);
+void setup_colors(void);
+void draw_time_playing(void);
+void draw_titles(void);
+void draw_time_scanning(void);
+void adjust_status(void);
+gint volume_changed(GtkWidget *widget, gpointer *data);
+gint launch_gmix(GtkWidget *widget, GdkEvent *event, gpointer data);
+gint fast_timer(gpointer *data);
+void setup_time_display(GtkWidget *table);
+void setup_fonts(void);
+void init_window(void);
+GtkWidget* make_button_with_pixmap(char *pic, GtkSignalFunc func,
+				   guint key,  gchar *tooltip);
+GtkWidget* make_button_stock(char *stock, GtkSignalFunc func,
+			     guint key, gchar *tooltip);
+void reload_info(int signal);
+void exit_action(void);
+void start_action(void);
+void setup_keys(void);
 
-void callback (GtkWidget *widget, gpointer *data)
+/* functions */
+gint roll_timer(gpointer *data)
 {
-    if( cd.isplayable ) tcd_gettime(&cd);
-    switch( (int)data )
-    {
-    case PLAY:
-	if( cd.sc.cdsc_audiostatus == CDROM_AUDIO_PAUSED )
-	    tcd_pausecd(&cd);
-	else 
-	    tcd_playtracks( &cd,cd.first_t,cd.last_t );
-	cd.repeat_track = cd.cur_t;
-	break;
-    case PAUSE:
-	tcd_pausecd(&cd);
-	break;
-    case STOP:
-	tcd_stopcd(&cd);
-	cd.play_method = NORMAL;
-	break;	                                                                                                        
-    case EJECT:
-	tcd_ejectcd(&cd);
-	cd.play_method = NORMAL;
-	cd.repeat_track = -1;
-	/* SDH: Make sure play/pause state change is noticed */
-	cd.sc.cdsc_audiostatus = -1;
-	break;
-    case FF:
-	tcd_play_seconds(&cd, 4);
-	cd.repeat_track = cd.cur_t;
-	break;
-    case RW:
-	tcd_play_seconds(&cd, -4);
-	cd.repeat_track = cd.cur_t;
-	break;
-    case NEXT_T:
-	if( cd.cur_t < cd.last_t ) 
-	{
-	    cd.cur_t++;
-	    tcd_playtracks( &cd,cd.cur_t, cd.last_t );
-	    if( cd.play_method==REPEAT_TRK )
-		cd.repeat_track = cd.cur_t;
-	}
-	break;
-    case PREV_T:
-	if( cd.cur_t > cd.first_t ) 
-	{
-	    if( (cd.t_sec+(cd.t_min*60)) < 10 )
-		cd.cur_t--;
-	    tcd_playtracks( &cd,cd.cur_t, cd.last_t );
-	    
-	    if( cd.play_method==REPEAT_TRK )
-		cd.repeat_track = cd.cur_t;
-	}
-	else
-	    tcd_playtracks( &cd, cd.cur_t, cd.last_t );
-	break;
-    case TRACKLIST:
-	edit_window();
-	break;
-    case QUIT:
-	delete_event(NULL,NULL);
-	break;		
-    case ABOUT:
-	about_cb(NULL,NULL);
-	break;	
-    case CDDB:
-	gcddb();
-	break;
-    case PROPS:
-	properties_cb(NULL, NULL);
-    default:
-    }
+    tcd_play_seconds(&cd, GPOINTER_TO_INT(data));
+    tcd_gettime(&cd);
     draw_status();
+    return TRUE;
 }
 
-void delete_event (GtkWidget *widget, gpointer *data)
+gint release_timer(gpointer *data)
 {
-    gtk_main_quit();
+    roll_t = gtk_timeout_add(35, (GtkFunction)roll_timer, data);
+    release_t = 0;
+    return FALSE;
 }
 
-GtkWidget* make_button( char *title, GtkWidget *box, int func, gchar *tooltip )
+int skip_cb(GtkWidget *widget, GdkEvent *event, gpointer *data)
 {
-    GtkWidget *button;
-    
-    button = gtk_button_new_with_label (title);
-    gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
-    gtk_widget_show (button);
-    Connect( button, func );
-    
-    gtk_tooltips_set_tip( tooltips, button, tooltip, "" );
-    
-    return button;
-}	                        
+    if(event->type == GDK_BUTTON_PRESS)
+    {
+	release_t = gtk_timeout_add(750, (GtkFunction)release_timer, data);
+    }
+    else if(event->type == GDK_BUTTON_RELEASE)
+    {
+	if(release_t)
+	{
+	    gtk_timeout_remove(release_t);
+	    switch(GPOINTER_TO_INT(data))
+	    {
+		/* +track */
+	    case 2:
+		if(cd.cur_t < cd.last_t)
+		{   
+		    cd.cur_t++;
+		    tcd_playtracks(&cd,cd.cur_t, cd.last_t);
+		    if(cd.play_method==REPEAT_TRK)
+			cd.repeat_track = cd.cur_t;
+		}
+		break;
+		/* -track */
+	    case -2:
+		if( cd.cur_t > cd.first_t )
+		{
+		    if( (cd.t_sec+(cd.t_min*60)) < 10 )
+			cd.cur_t--;
+		    tcd_playtracks( &cd,cd.cur_t, cd.last_t );
+				                                                             
+		    if( cd.play_method==REPEAT_TRK )
+			cd.repeat_track = cd.cur_t;
+		}
+		break;
+	    }
+	}   
+	if(roll_t)
+	    gtk_timeout_remove(roll_t);
+			
+	release_t = 0;
+	roll_t = 0;
+    }
+	
+    return FALSE;	
+}
 
-GtkWidget* make_button_with_pixmap( char *pic, GtkWidget *box, int func,
-				    gint expand, gint fill, gchar *tooltip )
+GtkWidget* make_button_with_pixmap(char *pic, GtkSignalFunc func,
+				   guint key ,  gchar *tooltip)
 {
     GtkWidget *button;
     GtkWidget *pixmap;
     char tmp[256];
     char *name;
-	
-    sprintf( tmp, "tcd/%s.xpm", pic );
-    name = gnome_pixmap_file (tmp);
+    
+    g_snprintf( tmp, 255, "tcd/%s.xpm", pic);
+    name = gnome_pixmap_file(tmp);
 #ifdef DEBUG
     g_print( "loading: %s\n", name );
 #endif
     pixmap = gnome_pixmap_new_from_file(name);
-    g_free (name);
-	
+    g_free(name);
+    
     button = gtk_button_new();
     gtk_container_add( GTK_CONTAINER(button), pixmap );
-    gtk_box_pack_start( GTK_BOX (box), button, expand, fill, 0 );
-	
-    if( func != -1 )
-	gtk_signal_connect(GTK_OBJECT (button), "clicked", \
-			   GTK_SIGNAL_FUNC (callback), (gpointer*)func );
-	
-    gtk_tooltips_set_tip( tooltips, button, tooltip, "" );
-	
-    gtk_widget_show_all(button);
+
+    if(key != -1)
+	add_key_binding(button, "clicked", tooltip, key);
+
+    if(func)
+	gtk_signal_connect(GTK_OBJECT (button), "clicked",
+			   GTK_SIGNAL_FUNC(func), NULL);
+    
+    gtk_tooltips_set_tip(tooltips, button, tooltip, "");
+    
     return button;
 }	                        
 
-void update_changer_buttons( void )
+GtkWidget* make_button_stock(char *stock, GtkSignalFunc func,
+			     guint key, gchar *tooltip)
 {
-    int i;
-    for( i=0; i < cd.nslots; i++ )
-    {
-	if( i == cd.cur_disc )
-	    gtk_widget_set_sensitive(GTK_WIDGET(changer_buttons[i]), FALSE );
-	else 
-	    gtk_widget_set_sensitive(GTK_WIDGET(changer_buttons[i]), TRUE );
-    }
-}	
-
-gint changer_callback( GtkWidget *widget, gpointer *data )
-{
-    tcd_close_disc(&cd);
-    tcd_change_disc( &cd, (int)data );
-    tcd_init_disc(&cd,create_warn);
+    GtkWidget *button;
+    GtkWidget *pixmap;
     
-    cd.play_method = NORMAL;
-    return 1;
-}
+    pixmap = gnome_stock_pixmap_widget_new(window,stock);
+    button = gtk_button_new();
+    gtk_container_add(GTK_CONTAINER(button), pixmap);
+
+    if(key != -1)
+	add_key_binding(button, "clicked", tooltip, key);
+
+    if(func)
+	gtk_signal_connect(GTK_OBJECT (button), "clicked",
+			   GTK_SIGNAL_FUNC(func), NULL);
+	
+    gtk_tooltips_set_tip( tooltips, button, tooltip, "" );
+	
+    return button;
+}	                        
 
 GtkWidget* make_changer_buttons( void )
 {
     GtkWidget *box;
+    GtkStyle *style;
     char tmp[5];
     int i;
     
-    box = gtk_hbox_new( FALSE, 0 );
-    changer_buttons = g_malloc( sizeof(GtkWidget)*cd.nslots );
+    box = gtk_hbox_new(FALSE, 0);
+    changer_buttons = (GtkWidget**)g_new(GtkWidget, cd.nslots);
     
-    for( i=0; i < cd.nslots && i < 12; i++ )
+    for(i=0; i < cd.nslots && i < 12; i++)
     {
-	sprintf( tmp, "%d", i+1 );
-	changer_buttons[i] = gtk_button_new_with_label( tmp );
-	gtk_widget_show( changer_buttons[i] );
-	gtk_box_pack_start( GTK_BOX(box), changer_buttons[i], FALSE, FALSE, 0 );
-	gtk_signal_connect( GTK_OBJECT(changer_buttons[i]), "clicked", \
-			    GTK_SIGNAL_FUNC(changer_callback), (gpointer*)i );
+	g_snprintf( tmp, 4, "%d", i+1 );
+	changer_buttons[i] = gtk_button_new_with_label(tmp);
+
+	style = gtk_widget_get_style(GTK_BUTTON(changer_buttons[i])->child);
+	gdk_font_unref(style->font);
+	style->font = gdk_font_load("-misc-fixed-*-*-*-*-10-*-*-*-*-*-*-*");
+	gdk_font_ref(style->font);
+	gtk_widget_set_style(GTK_BUTTON(changer_buttons[i])->child, style);
+	gdk_font_unref(style->font);
+	gtk_style_unref(style);
+
+	gtk_box_pack_start(GTK_BOX(box), changer_buttons[i], TRUE, TRUE, 0);
+	gtk_signal_connect(GTK_OBJECT(changer_buttons[i]), "clicked", \
+			   GTK_SIGNAL_FUNC(changer_cb), GINT_TO_POINTER(i));
     }
     return box;
 }
@@ -301,156 +293,107 @@ static gint button_press (GtkWidget *widget, GdkEvent *event)
     return FALSE;
 }
 
-GtkWidget* make_row1( void )
+GtkWidget *make_small_buttons(void)
 {
-    GtkWidget *box, *handle, *pixmap;
+    GtkWidget *b1, *b2, *b3;
+    GtkWidget *table;
     
-    box = gtk_hbox_new( TRUE, 0 );
-    handle = gtk_handle_box_new();
-    
-    playbutton = make_button_with_pixmap( "play", box, -1, TRUE, TRUE, TT_PLAY );
-    status_changed();
-    
-    make_button_with_pixmap( "stop", box, STOP, TRUE, TRUE, TT_STOP );
-    make_button_with_pixmap( "eject", box, EJECT, TRUE, TRUE, TT_EJECT );
-    
-    propsbutton = gtk_button_new();
-    pixmap = gnome_stock_pixmap_widget( window, GNOME_STOCK_PIXMAP_PREFERENCES );
-    gtk_container_add(GTK_CONTAINER(propsbutton), pixmap );
-    gtk_signal_connect(GTK_OBJECT(propsbutton), "clicked",
-		       GTK_SIGNAL_FUNC(callback), (gpointer*)PROPS );
-    gtk_box_pack_start(GTK_BOX(box), propsbutton, TRUE, TRUE, 0);
-    gtk_tooltips_set_tip( tooltips, propsbutton, TT_PROPS, "" );
-    
-    if( props.handle )
-    {
-	gtk_widget_show(box);
-	gtk_container_add( GTK_CONTAINER(handle), box );
-	row1=handle;
-    }
-    else
-	row1=box;
-    return row1;
+    table = gtk_table_new(TRUE, 6,1);
+
+    b1 = make_button_stock(GNOME_STOCK_PIXMAP_PROPERTIES, edit_window, 'T', _("Open track editor"));
+    b2 = make_button_stock(GNOME_STOCK_PIXMAP_PREFERENCES, preferences, -1, _("Preferences"));
+    b3 = make_button_stock(GNOME_STOCK_PIXMAP_QUIT, quit_cb, 'Q', _("Quit"));
+
+    gtk_table_attach_defaults(GTK_TABLE(table), b1, 0, 2, 0, 1);
+    gtk_table_attach_defaults(GTK_TABLE(table), b2, 2, 4, 0, 1);
+    gtk_table_attach_defaults(GTK_TABLE(table), b3, 4, 6, 0, 1);
+
+    return table;
 }
 
-GtkWidget* make_row2( void )
+GtkWidget* create_buttons(void)
 {
-    GtkWidget *box, *handle;
-    box = gtk_hbox_new( TRUE, 0 );
-    handle = gtk_handle_box_new();
-
-    make_button_with_pixmap("prev_t", box, PREV_T, TRUE, TRUE, TT_PREV_TRACK );
-    make_button_with_pixmap("rw", box, RW, TRUE, TRUE, TT_REWIND );
-    make_button_with_pixmap("ff", box, FF, TRUE, TRUE, TT_FF );
-    make_button_with_pixmap("next_t", box, NEXT_T, TRUE, TRUE, TT_NEXT_TRACK );
-
-    if( props.handle )
-    {
-	gtk_widget_show(box);
-	gtk_container_add( GTK_CONTAINER(handle), box );
-	row2=handle;
-    }
-    else
-	row2=box;
-    return row2;
-}
-
-GtkWidget* make_row3( void )
-{
-    GtkWidget *box, *bbox;
-    GtkWidget *pixmap, *handle;
+    GtkWidget *table;
+    GtkWidget *b1, *b2, *b3;
+    GtkWidget *pixmap;
+    GtkWidget *rw, *ff;
     char *name;
-	
-    box = gtk_hbox_new( TRUE, 0 );
-    bbox = gtk_vbox_new( FALSE, 0 );
-    handle = gtk_handle_box_new();
 
-    cddbbutton = make_button_with_pixmap( "cddb", box, CDDB, TRUE, TRUE,  TT_CDDB );
-    trackeditor =make_button_with_pixmap( "edit", box, TRACKLIST, TRUE, TRUE, TT_TRACKED );
+    table = gtk_table_new(TRUE, 3, 4);
 
+/* TOP ROW */ 
+     
+    playbutton = make_button_with_pixmap("play", NULL, 'P', _("Play"));
+    status_changed();
+    b1 = playbutton;
+    
+    b2 = make_button_with_pixmap( "stop", stop_cb, 'S', _("Stop") );
+    b3 = make_button_with_pixmap( "eject", eject_cb, 'E', _("Eject") );
+    
+    gtk_table_attach_defaults(GTK_TABLE(table), b1, 0, 1, 0, 1);
+    gtk_table_attach_defaults(GTK_TABLE(table), b2, 1, 2, 0, 1);
+    gtk_table_attach_defaults(GTK_TABLE(table), b3, 2, 3, 0, 1);
+
+/* MIDDLE ROW */
+    rw = make_button_with_pixmap("rw", NULL, -1, _("Skip backwards"));
+    ff = make_button_with_pixmap("ff", NULL, -1, _("Skip forwards"));
+
+    gtk_widget_set_events(rw, GDK_BUTTON_PRESS_MASK
+			  | GDK_BUTTON_RELEASE_MASK);
+    gtk_widget_set_events(ff, GDK_BUTTON_PRESS_MASK
+			  | GDK_BUTTON_RELEASE_MASK);
+
+    add_key_binding(rw, "clicked", _("Skip backwards"), '-');
+    add_key_binding(rw, "clicked", _("Skip forwards"), '+');
+
+    gtk_signal_connect(GTK_OBJECT(ff), "event",
+		       GTK_SIGNAL_FUNC(skip_cb), GINT_TO_POINTER(2));
+    gtk_signal_connect(GTK_OBJECT(rw), "event",
+		       GTK_SIGNAL_FUNC(skip_cb), GINT_TO_POINTER(-2));
+
+/* Create goto button  */
     gotobutton = gtk_button_new();
     name = gnome_pixmap_file("tcd/goto.xpm");
     pixmap = gnome_pixmap_new_from_file(name);
-    g_free (name);
-    gtk_box_pack_start( GTK_BOX(bbox), pixmap, FALSE, FALSE, 0 );
-    gtk_container_add( GTK_CONTAINER(gotobutton), bbox);
-    gtk_box_pack_start( GTK_BOX(box), gotobutton, TRUE, TRUE, 0);
-	
-    make_button_with_pixmap( "power", box, QUIT, TRUE, TRUE, N_("Quit") );
+    g_free(name);
+    gtk_container_add(GTK_CONTAINER(gotobutton), pixmap);
 
-    gtk_widget_show_all(box);
-	
-    if( props.handle )
-    {
-	gtk_widget_show(box);
-	gtk_container_add( GTK_CONTAINER(handle), box );
-	row3=handle;
-    }
-    else
-	row3=box;
-    return row3;
+    gtk_table_attach_defaults(GTK_TABLE(table), rw, 0, 1, 1, 2);
+    gtk_table_attach_defaults(GTK_TABLE(table), ff, 1, 2, 1, 2);
+    gtk_table_attach_defaults(GTK_TABLE(table), gotobutton, 2, 3, 1, 2);
+    gtk_table_attach_defaults(GTK_TABLE(table), make_small_buttons(), 0, 3, 2, 3);
+
+    return table;
 }
 
 void setup_colors( void )
 {
+    char tmp[16];
     GdkColormap *colormap;
     
     colormap = gtk_widget_get_colormap(status_area);
     
     gdk_color_parse("#555555", &darkgrey);
     gdk_color_alloc(colormap, &darkgrey);
-    
-    gdk_color_parse(props.trackcolor, &track_color);
+
+    g_snprintf(tmp, 15, "#%02X%02X%02X", 
+	       prefs.trackcolor_r,
+	       prefs.trackcolor_g,
+	       prefs.trackcolor_b);
+
+    gdk_color_parse(tmp, &track_color);
     gdk_color_alloc(colormap, &track_color);
     draw_status();
 }
 
-void setup_pixmaps( void )
-{
-    int r,g,b;
-    GdkPixmap *pm;
-    GdkBitmap *bm;
-    GdkImlibImage *im;
-    gchar tmp[128];
-    int i;
-    char *name;
-
-    sscanf( props.statuscolor, "#%02x%02x%02x", &r,&g,&b );
-	        
-    for( i=0; i < PLAY_METHOD_END-1; i++ )
-    {
-	sprintf( tmp, "tcd/%s", play_methods[i] );
-	name = gnome_pixmap_file(tmp);
-	im = gdk_imlib_load_image(name);
-	g_free (name);
-/*
-  gdk_imlib_get_image_red_modifier(im, &m);
-  m.brightness = r*256;
-  gdk_imlib_set_image_red_modifier(im, &m);
-
-  gdk_imlib_get_image_green_modifier(im, &m);
-  m.brightness = g*256;
-  gdk_imlib_set_image_green_modifier(im, &m);
-
-  gdk_imlib_get_image_blue_modifier(im, &m);
-  m.brightness = b*256;
-  gdk_imlib_set_image_blue_modifier(im, &m);
-  */
-	gdk_imlib_render(im, im->rgb_width, im->rgb_height);
-	pm = gdk_imlib_move_image(im);
-	bm = gdk_imlib_move_mask(im);
-	play_method_pixmap[i] = gtk_pixmap_new(pm, bm);		
-    }
-}
-
-void draw_time_playing( GdkGC *gc )
+void draw_time_playing(void)
 {
     int pos, end, cur, min, sec;
-    switch( props.time_display )
+    switch( prefs.time_display )
     {
     case TRACK_E: /* track time ascending */
 	led_draw_time(status_db, status_area,
-		      32,4, cd.t_min, cd.t_sec);
+		      48,4, cd.t_min, cd.t_sec);
 	break;
     case TRACK_R: /* track time decending */
 	cur = cd.cur_pos_rel;
@@ -460,11 +403,11 @@ void draw_time_playing( GdkGC *gc )
 	min = pos/60;
 	sec = pos-(pos/60)*60;
 	led_draw_time(status_db, status_area,
-		      32,4, min, sec);
+		      48,4, min, sec);
 	break;
     case DISC_E: /* disc time ascending */
 	led_draw_time(status_db, status_area,
-		      32,4, cd.cd_min, cd.cd_sec);
+		      48,4, cd.cd_min, cd.cd_sec);
 	break;
     case DISC_R:
 	cur = cd.cur_pos_abs;
@@ -474,14 +417,14 @@ void draw_time_playing( GdkGC *gc )
 	min = pos/60;
 	sec = pos-(pos/60)*60;
 	led_draw_time(status_db, status_area,
-		      32, 4, min, sec);
+		      48, 4, min, sec);
 	break;
     default:
 	break;
     }		
 }
 
-void draw_titles( GdkGC *gc )
+void draw_titles(void)
 {
     int inc;
 
@@ -497,26 +440,22 @@ void draw_titles( GdkGC *gc )
     gtk_window_set_title( GTK_WINDOW(window), cd.trk[cd.cur_t].name );
 }
 
-void draw_time_scanning( GdkGC *gc )
+void draw_time_scanning(void)
 {
     gdk_gc_set_foreground( gc, &track_color );
     gdk_draw_text(status_db,tfont,gc,4,39+(tfont->ascent+tfont->descent)-2, 
-    		N_("(Scanning)"),
-		10);
-    gtk_window_set_title( GTK_WINDOW(window), N_("(Scanning)") );
+                  _("(Scanning)"),
+                  strlen( _("(Scanning)")));
+    gtk_window_set_title( GTK_WINDOW(window), _("(Scanning)") );
 }
 
-void draw_status( void )
+void draw_status(void)
 {
     char tmp[128];
-    GdkGC *gc;
 
     if( !configured )
 	return;
 
-    gc = gdk_gc_new( window->window );
-    gdk_gc_copy( gc, status_area->style->white_gc );
-
     /* Erase Rectangle */
     gdk_draw_rectangle( status_db, 
 			status_area->style->black_gc,
@@ -529,60 +468,57 @@ void draw_status( void )
 	switch( cd.sc.cdsc_audiostatus )
 	{
 	case CDROM_AUDIO_INVALID:
-	    strcpy(tmp, N_("No Disc"));
+	    strcpy(tmp, _("No Disc"));
 	    gdk_draw_text( status_db,tfont,gc,4,39,tmp, strlen(tmp) );
-	    draw_time_scanning(gc);
+	    draw_time_scanning();
 	    break;
 	case CDROM_AUDIO_PLAY:
 	    if( cd.isplayable ) /* we can't be playing if we can't play */
 	    {
-		draw_time_playing(gc);
-		draw_titles(gc);
+		draw_time_playing();
+		draw_titles();
 		led_draw_track(status_db, status_area, 
 			       4,4, cd.cur_t);
 	    }
 	    else
-		draw_time_scanning(gc);
+		draw_time_scanning();
 	    break;
 	case CDROM_AUDIO_PAUSED:
-	    draw_titles(gc);
+	    draw_titles();
 	    break;
 	case CDROM_AUDIO_COMPLETED:
 	case CDROM_AUDIO_NO_STATUS:
 	    led_stop_time(status_db, status_area, 34,4 );
 	    led_stop_track(status_db, status_area, 4,4 );
-	    if( cd.isplayable )
-		draw_titles(gc);
+	    if(cd.isplayable)
+		draw_titles();
 	    else
-		draw_time_scanning(gc);
+		draw_time_scanning();
 	    break;
 	case CDROM_AUDIO_ERROR:
-	    strcpy( tmp,N_("Error") );
+	    strcpy( tmp, _("Error") );
 	    gdk_draw_text( status_db,tfont,gc,4,39,tmp, strlen(tmp) );
-	    draw_time_scanning(gc);
+	    draw_time_scanning();
 	    break;
 	default:
 	    break;
 	}
 	if(cd.isplayable)
 	{		
-	    gdk_draw_pixmap(status_db,
-			    status_area->style->fg_gc[GTK_WIDGET_STATE(status_area)],
-			    GTK_PIXMAP(play_method_pixmap[cd.play_method])->pixmap,
-			    0, 0,
-			    80, -2,
-			    24, 24);
-	    gdk_gc_set_foreground( gc, &darkgrey );
-	    gdk_draw_text( status_db,tfont,gc,32,26, 
-			   display_types[props.time_display], 
-			   strlen(display_types[props.time_display]));
+	    gdk_gc_set_foreground(gc, &darkgrey);
+	    gdk_draw_text( status_db,sfont,gc,48,26, 
+			   display_types[prefs.time_display], 
+			   strlen(display_types[prefs.time_display]));
+	    gdk_draw_text( status_db,sfont,gc,2,26, 
+			   play_types[cd.play_method], 
+			   strlen(play_types[cd.play_method]));
 	}
     }
     else
     {
-	strcpy( tmp, cd.errmsg );
-	gdk_draw_text( status_db,tfont,gc,
-		       4, 39, tmp, strlen(tmp) );
+	strncpy(tmp, cd.errmsg, sizeof(tmp));
+	gdk_draw_text(status_db,tfont,gc,
+		      4, 39, tmp, strlen(tmp));
     }
 
     /* Finally, update the display */
@@ -593,7 +529,6 @@ void draw_status( void )
 		    0, 0,
 		    status_area->allocation.width,
 		    status_area->allocation.height);
-    gdk_gc_destroy(gc);
 }
 
 void adjust_status(void)
@@ -620,7 +555,7 @@ void adjust_status(void)
 gint slow_timer( gpointer *data )
 {
     if( cd.cddb_id != cur_goto_id )
-	make_gotomenu();
+	make_goto_menu();
 
     if( cd.err || !cd.isplayable )
     {
@@ -636,21 +571,22 @@ gint slow_timer( gpointer *data )
     }
     tcd_gettime(&cd);
     adjust_status();
-    draw_status();
+    draw_status(); 
     return 1;
 }
 
 void status_changed(void)
 {
-    if( old_status != cd.sc.cdsc_audiostatus)
+    if(old_status != cd.sc.cdsc_audiostatus)
     {
 	GtkWidget *pixmap;
+	GtkSignalFunc func;
 	char tmp[256];
 	char *name;
-
+	
 	old_status = cd.sc.cdsc_audiostatus;
-	sprintf( tmp, "tcd/%s.xpm", 
-		 (old_status==CDROM_AUDIO_PLAY)?"pause":"play" );
+	g_snprintf(tmp, 255, "tcd/%s.xpm", 
+		    (old_status==CDROM_AUDIO_PLAY)?"pause":"play");
 
 	gtk_widget_destroy(GTK_BUTTON(playbutton)->child);
 	GTK_BUTTON(playbutton)->child = NULL;
@@ -661,20 +597,24 @@ void status_changed(void)
 	gtk_widget_show(pixmap);
 	gtk_container_add( GTK_CONTAINER(playbutton), pixmap );
 
-	if( playid > 0 )
+	if(old_status == CDROM_AUDIO_PLAY)
+	    func = GTK_SIGNAL_FUNC(pause_cb);
+	else
+	    func = GTK_SIGNAL_FUNC(play_cb);
+	
+	if(playid > 0)
 	    gtk_signal_disconnect(GTK_OBJECT(playbutton), playid);
-	playid = gtk_signal_connect(GTK_OBJECT(playbutton), "clicked", \
-				    GTK_SIGNAL_FUNC (callback),
-				    (old_status==CDROM_AUDIO_PLAY)?(gpointer*)PAUSE:(gpointer*)PLAY );
+
+	playid = gtk_signal_connect(GTK_OBJECT(playbutton), "clicked",
+				    func, NULL);
     }
 }
 
 gint volume_changed( GtkWidget *widget, gpointer *data )
 {
-    if( !data )
+    if(!data)
     {
-	cd.volume = (int)floor(GTK_ADJUSTMENT(vol)->value);
-	if( cd.isplayable ) tcd_gettime(&cd);
+	tcd_set_volume(&cd, (int)floor(GTK_ADJUSTMENT(vol)->value));
     }
     draw_status();
     return 1;
@@ -682,41 +622,33 @@ gint volume_changed( GtkWidget *widget, gpointer *data )
 
 gint launch_gmix( GtkWidget *widget, GdkEvent *event, gpointer data ) 
 {
-    if ( event->type==GDK_2BUTTON_PRESS )
+    if(event->type==GDK_2BUTTON_PRESS)
         if (fork()==0)
             execlp("gmix","",NULL); 
 
-     return FALSE;
+    return FALSE;
 }
 
-gint gototrack( GtkWidget *widget, gpointer *data )
+void make_goto_menu()
 {
-    tcd_playtracks( &cd, (int)data, cd.last_t );
-    cd.repeat_track = (int)data;
-    return 1;
-}
-
-void make_gotomenu()
-{
-    char buf[128];
+    char buf[TRK_NAME_LEN];
     int i;
     GtkWidget *item;
-    gotomenu = NULL;
+
     gotomenu = gtk_menu_new();
 
     for( i=1; i <= cd.last_t; i++ )
     {
-	sprintf( buf, "%2d - %s", i, cd.trk[C(i)].name );
-
+	g_snprintf(buf, TRK_NAME_LEN-1, "%2d - %s", i, cd.trk[C(i)].name);
 	item = gtk_menu_item_new_with_label(buf);
-	gtk_menu_append(GTK_MENU (gotomenu), item);
-	gtk_signal_connect_object(GTK_OBJECT(item), "activate",
-				  GTK_SIGNAL_FUNC(gototrack), (gpointer)i );
 	gtk_widget_show(item);
+	gtk_menu_append(GTK_MENU(gotomenu), item);
+	gtk_signal_connect_object(GTK_OBJECT(item), "activate",
+				  GTK_SIGNAL_FUNC(goto_track_cb), GINT_TO_POINTER(i));
     }
-    if( gotoi ) gtk_signal_disconnect( GTK_OBJECT(gotobutton), gotoi );
+    if(gotoi) gtk_signal_disconnect(GTK_OBJECT(gotobutton), gotoi);
     gotoi = gtk_signal_connect_object(GTK_OBJECT(gotobutton), "event",
-				      GTK_SIGNAL_FUNC (button_press), GTK_OBJECT(gotomenu));
+				      GTK_SIGNAL_FUNC(button_press), GTK_OBJECT(gotomenu));
 
     cur_goto_id = cd.cddb_id;
 }
@@ -725,8 +657,8 @@ gint fast_timer( gpointer *data )
 {
     tcd_gettime(&cd);
     status_changed();
-    if( (cd.play_method==REPEAT_TRK) && (cd.cur_t != cd.repeat_track) )
-	tcd_playtracks( &cd, cd.repeat_track, -1 );
+    if((cd.play_method==REPEAT_TRK) && (cd.cur_t != cd.repeat_track))
+	tcd_playtracks(&cd, cd.repeat_track, -1);
     return 1;
 }
 
@@ -735,7 +667,7 @@ static gint status_configure_event(GtkWidget *widget, GdkEventConfigure *event)
     static int first=TRUE;
     configured = TRUE;	
     if( status_db )
-	gdk_pixmap_unref (status_db);
+	gdk_pixmap_unref(status_db);
 		
     status_db = gdk_pixmap_new( widget->window, 
 				widget->allocation.width,
@@ -760,7 +692,7 @@ static gint status_configure_event(GtkWidget *widget, GdkEventConfigure *event)
 	status_height = status_area->allocation.height;
 	first=FALSE;
     }
-    GTK_ADJUSTMENT(vol)->value = (double)cd.volume;
+    GTK_ADJUSTMENT(vol)->value = (double)tcd_get_volume(&cd);
     gtk_signal_emit_by_name(GTK_OBJECT(vol),"value_changed", "no_update");
     return TRUE;
 }
@@ -787,14 +719,12 @@ static gint status_click_event(GtkWidget *widget, GdkEvent *event)
     if( event->type == GDK_BUTTON_PRESS )
     {
 	if( e->button > 1 )
-	{
-	    about_cb(NULL, NULL);
-	    return TRUE;
-	}
-	if( x > 80 &&
+	    return FALSE;
+
+	if( x > 2 &&
 	    y > 0 &&
-	    x < 104 &&
-	    y < 24 )
+	    x < 48 &&
+	    y < 32 )
 	{
 	    cd.play_method++;
 	    if( cd.play_method > NORMAL )
@@ -805,14 +735,14 @@ static gint status_click_event(GtkWidget *widget, GdkEvent *event)
 	    draw_status();
 	    return TRUE;
 	}
-	if( x > 28 &&
+	if( x > 48 &&
 	    y > 0 &&
-	    x < 79 &&
-	    y < 27 )
+	    x < 100 &&
+	    y < 32 )
 	{
-	    props.time_display++;
-	    if( props.time_display >= TIME_LAST )
-		props.time_display = TIME_FIRST+1;
+	    prefs.time_display++;
+	    if( prefs.time_display >= TIME_LAST )
+		prefs.time_display = TIME_FIRST+1;
 	    draw_status();
 	    return TRUE;
 	}
@@ -821,9 +751,10 @@ static gint status_click_event(GtkWidget *widget, GdkEvent *event)
     return FALSE;
 }
 
-void setup_time_display( void )
+void setup_time_display(GtkWidget *table)
 {
     GtkWidget *handle1, *frame;
+    GtkWidget *box;
 
     vol = gtk_adjustment_new (0.0, 0.0, 256.0, 0.1, 1.0, 1.0);
     volume = gtk_hscale_new(GTK_ADJUSTMENT(vol));
@@ -834,10 +765,6 @@ void setup_time_display( void )
     gtk_signal_connect( GTK_OBJECT(volume), "button_press_event",
 			(GtkSignalFunc)launch_gmix, NULL);
 
-#ifdef TCD_CHANGER_ENABLED
-    changer_box = make_changer_buttons();
-#endif
-
     status_area = gtk_drawing_area_new();
     gtk_signal_connect( GTK_OBJECT (status_area), "expose_event",
 			(GtkSignalFunc)status_expose_event, NULL);
@@ -847,8 +774,6 @@ void setup_time_display( void )
 			(GtkSignalFunc)status_click_event, NULL);
     gtk_widget_set_usize( status_area, 102, 60 );
 
-    gtk_tooltips_set_tip( tooltips, status_area, TT_TIME, "" );
-        
     gtk_widget_set_events (status_area, GDK_EXPOSURE_MASK
 			   | GDK_LEAVE_NOTIFY_MASK
 			   | GDK_BUTTON_PRESS_MASK
@@ -858,45 +783,26 @@ void setup_time_display( void )
     frame = gtk_frame_new(NULL);
     gtk_frame_set_shadow_type( GTK_FRAME(frame), GTK_SHADOW_IN );
     gtk_container_add(GTK_CONTAINER(frame), status_area);
-    status_table = gtk_vbox_new( FALSE, 2 );
+    status_table = gtk_vbox_new(FALSE, 2);
 
+    box = gtk_hbox_new(FALSE, 2);
 #ifdef TCD_CHANGER_ENABLED
-    gtk_box_pack_start( GTK_BOX(main_box), changer_box, FALSE, FALSE, 0 );
+    gtk_box_pack_start(GTK_BOX(box), make_changer_buttons(), FALSE, FALSE, 0);
 #endif
-    gtk_box_pack_end( GTK_BOX(status_table), volume, FALSE, FALSE, 2 );
+    gtk_box_pack_start(GTK_BOX(box), volume, TRUE, TRUE, 0);
 
-    if( props.handle )
+    gtk_box_pack_end(GTK_BOX(status_table), box, FALSE, FALSE, 0);
+
+    if( prefs.handle )
     {
 	handle1 = gtk_handle_box_new();
 	gtk_container_add(GTK_CONTAINER(handle1), frame );
 	gtk_box_pack_start( GTK_BOX(status_table), handle1, TRUE, TRUE, 2 );
     }
     else
-	gtk_box_pack_start( GTK_BOX(status_table), frame, TRUE, TRUE, 2);
+	gtk_box_pack_start( GTK_BOX(status_table), frame, TRUE, TRUE, 0);
 		                
-    gtk_box_pack_start(GTK_BOX(main_box), status_table, TRUE, TRUE, 2);
-    gtk_widget_show_all(status_table);
-    return;
-}
-
-void setup_rows( void )
-{
-    sep = gtk_hseparator_new();
-
-    button_box = gtk_vbox_new( TRUE, 0 );
-
-    row = make_row1();
-
-    gtk_box_pack_start( GTK_BOX(button_box), row, TRUE, TRUE, 0 ); 
-	
-    row = make_row2();
-    gtk_box_pack_start( GTK_BOX(button_box), row, TRUE, TRUE, 0 );
-
-    row = make_row3();
-    gtk_box_pack_start( GTK_BOX(button_box), row, TRUE, TRUE, 0 );
-
-    gtk_box_pack_start( GTK_BOX(main_box), button_box, TRUE, TRUE, 0 );
-
+    gtk_box_pack_start(GTK_BOX(main_box), status_table, FALSE, FALSE, 2);
     return;
 }
 
@@ -904,19 +810,17 @@ void setup_fonts(void)
 {
     if( tfont )
 	gdk_font_unref(tfont);
-    if( sfont )
-	gdk_font_unref(sfont);
 
-    if(props.statusfont)
-	sfont = gdk_font_load( props.statusfont );
-    else
-	sfont = NULL;
-
-    if(props.trackfont)
-	tfont = gdk_font_load( props.trackfont );
+    if(prefs.trackfont)
+	tfont = gdk_font_load( prefs.trackfont );
     else
 	tfont = NULL;
 
+    if(sfont)
+	gdk_font_unref(sfont);
+    
+    sfont = gdk_font_load("-misc-fixed-*-*-*-*-12-*-*-*-*-*-*-*");
+    
     tfont_height = (tfont->ascent+tfont->descent)*3;
 }
 
@@ -924,77 +828,142 @@ void init_window(void)
 {
     setup_fonts();
 
-    window = gnome_app_new( "gtcd", _("TCD 2.0") );
-    gtk_window_set_title( GTK_WINDOW(window), PACKAGE" "VERSION" " );
-    gtk_window_set_wmclass( GTK_WINDOW(window), "main_window","gtcd" );
+    window = gnome_app_new("gtcd", "TCD 2.0");
+    gtk_window_set_title(GTK_WINDOW(window), PACKAGE" "VERSION" ");
+    gtk_window_set_wmclass( GTK_WINDOW(window), "main_window","gtcd");
     gtk_window_set_policy(GTK_WINDOW(window), TRUE, FALSE, TRUE);
 
-    gtk_signal_connect( GTK_OBJECT(window), "delete_event",
-			GTK_SIGNAL_FUNC(delete_event), NULL);
+    gtk_signal_connect(GTK_OBJECT(window), "delete_event",
+		       GTK_SIGNAL_FUNC(quit_cb), NULL);
 
-    gtk_container_border_width( GTK_CONTAINER(window), 4 );
+    gtk_container_border_width(GTK_CONTAINER(window), 3);
     gtk_widget_realize(window);
 
-    main_box = gtk_hbox_new( FALSE, 5 );
+    gc = gdk_gc_new(window->window);
+    gdk_gc_copy(gc, window->style->white_gc);
+
+    main_box = gtk_hbox_new(FALSE, 3);
 
     tooltips = gtk_tooltips_new();
-    if( props.tooltip )
+    if( prefs.tooltip )
 	gtk_tooltips_enable(tooltips);
     else
 	gtk_tooltips_disable(tooltips);
 }
 
-void create_warn( char *message_text, char *type )
+void create_warning(char *message_text, char *type)
 {
     gtk_widget_show(gnome_message_box_new(message_text, type,
 					  GNOME_STOCK_BUTTON_OK, NULL));
 }
-	
+
+void reload_info(int signal)
+{
+    tcd_close_disc(&cd);
+    tcd_init_disc(&cd, create_warning);
+    make_goto_menu();
+    draw_status();
+}
+
+void exit_action(void)
+{
+    switch(prefs.exit_action)
+    {
+    case StopPlaying:
+	tcd_stopcd(&cd);
+	break;
+    case OpenTray:
+	tcd_ejectcd(&cd);
+	break;
+    case CloseTray:
+	ioctl(cd.cd_dev, CDROMCLOSETRAY);
+	break;
+    case DoNothing:
+    default:
+	break;
+    }
+}	
+
+void start_action(void)
+{
+    if(prefs.close_tray_on_start)
+	ioctl(cd.cd_dev, CDROMCLOSETRAY);
+    switch(prefs.start_action)
+    {
+    case StartPlaying:
+	tcd_playtracks(&cd, cd.first_t, cd.last_t);
+	break;
+    case StopPlaying:
+	tcd_stopcd(&cd);
+	break;
+    case DoNothing:
+    default:
+	break;
+    }
+}	
+
+void setup_keys()
+{
+    accel = gtk_accel_group_get_default();
+}    
+
 int main (int argc, char *argv[])
 {
+    GtkWidget *table;
     char *homedir;
-    char rcfile[64];
+    char rcfile[256];
 
     argp_program_version = VERSION;
 
-   bindtextdomain(PACKAGE, GNOMELOCALEDIR);
-   textdomain(PACKAGE);
-                
+    bindtextdomain(PACKAGE, GNOMELOCALEDIR);
+    textdomain(PACKAGE);
 
-    gnome_init( "gtcd", NULL, argc, argv, 0, NULL );
+    gnome_init("gtcd", NULL, argc, argv, 0, NULL);
 
     homedir = getenv("HOME");
-    sprintf( rcfile, "%s/.tcd/gtcdrc", homedir );
-    gtk_rc_parse( rcfile );
+    g_snprintf(rcfile, 255, "%s/.tcd/gtcdrc", homedir);
+    gtk_rc_parse(rcfile);
 
     cd.play_method = NORMAL;        
 
-    load_properties(&props);
-    cd.cdpath = props.cddev;
+    load_prefs(&prefs);
+    cd.cdpath = prefs.cddev;
 	
-    tcd_init_disc(&cd, (WarnFunc)create_warn);
-        
-    init_window();
-    setup_rows();
-    make_gotomenu();
-    setup_time_display();
-    setup_colors();
-    setup_pixmaps();
-    led_init(window);
-	
-    /* Initialize some timers */
-    if( cd.isplayable ) tcd_gettime(&cd);
+    tcd_init_disc(&cd, (WarnFunc)create_warning);
 
+    start_action();
+    signal(SIGUSR1, reload_info);
+    
+    setup_keys();
+    init_window();
+
+    table = create_buttons();
+    gtk_box_pack_start(GTK_BOX(main_box), table, FALSE, FALSE, 0);
+    
+    make_goto_menu();
+    setup_time_display(table);
+    setup_colors();
+    led_init(window);
+
+    setup_popup_menu(status_area, &cd);
+    
+    /* Initialize some timers */
+    if(cd.isplayable) tcd_gettime(&cd);
+    
     gtk_timeout_add(1000, (GtkFunction)slow_timer, NULL);
-    gtk_timeout_add(250, (GtkFunction)fast_timer, NULL);
+    gtk_timeout_add(500, (GtkFunction)fast_timer, NULL);
     titlelabel_f = TRUE;
 	
-    gnome_app_set_contents( GNOME_APP(window), main_box );
+    gnome_app_set_contents(GNOME_APP(window), main_box);
     adjust_status();	
     gtk_widget_show_all(window);
 	
     gtk_main ();
-    save_properties(&props);
+    save_prefs(&prefs);
     gnome_config_sync();
+    gdk_gc_destroy(gc);
+    exit_action();
+    tcd_close_disc(&cd);
+
     return 0;
 }
