@@ -1,5 +1,3 @@
-#undef PIXMAPS
-
 #include <config.h>
 #include <gnome.h>
 #include <sys/types.h>
@@ -7,32 +5,33 @@
 #include <linux/cdrom.h>
 #include "linux-cdrom.h"
 
-#ifdef PIXMAPS
-#include "icons/data.xpm"
-#include "icons/music.xpm"
-#endif
+#include "gtcd_public.h"
 
 GtkWidget *trwin;
 
-#ifdef PIXMAPS
-GtkWidget *music, *data;
-#endif
+static void destroy_window(GtkWidget *widget, gboolean save);
+static void update_list(GtkWidget *list, int track);
+static void fill_list(GtkWidget *list);
+static void dtitle_changed(GtkWidget *widget, gpointer data);
+static void activate_entry(GtkWidget *widget, GtkWidget *list);
+static void select_row_cb(GtkCList *clist,
+		   gint row,
+		   gint column,
+		   GdkEventButton *event,
+		   GtkWidget *entry);
+static void next_entry(GtkWidget *widget, GtkWidget *list);
 
-extern cd_struct cd;
-
-void make_gotomenu();
- 
-void destroy_window (GtkWidget *widget, gboolean save)
+static void destroy_window (GtkWidget *widget, gboolean save)
 {
 	if( save )
 		tcd_writediskinfo(&cd);
         gtk_widget_destroy(trwin);
 	trwin = NULL;
-	make_gotomenu();
+	make_goto_menu();
 	return;
 }
 
-void update_list( GtkWidget *list, int track )
+static void update_list( GtkWidget *list, int track )
 {
 	gtk_clist_freeze(GTK_CLIST(list));
 	gtk_clist_set_text(GTK_CLIST(list), track-1, 2, cd.trk[track].name );
@@ -40,14 +39,14 @@ void update_list( GtkWidget *list, int track )
 	return;
 }
 
-void fill_list( GtkWidget *list )
+static void fill_list( GtkWidget *list )
 {
 	int i;
-	gchar *tmp[4];
+	char *tmp[4];
+	
+	for(i=0; i < 4; i++)
+		tmp[i] = g_malloc(255);
 
-	for( i=0; i < 3; i++ )
-		tmp[i] = malloc(255);
-		
 	gtk_clist_freeze(GTK_CLIST(list));
 
 	for( i=1; i <= cd.last_t; i++ )
@@ -55,29 +54,28 @@ void fill_list( GtkWidget *list )
 		g_snprintf(tmp[0], 255, "%d", i);
 		g_snprintf(tmp[1], 255, "%2d:%02d",
 			cd.trk[i].tot_min, cd.trk[i].tot_sec);
-		strcpy(tmp[2], cd.trk[i].name);
+		strncpy(tmp[2], cd.trk[i].name, 255);
 		tmp[3] = NULL;
 	
-		gtk_clist_append(GTK_CLIST(list), tmp );
+		gtk_clist_append(GTK_CLIST(list), tmp);
 		g_snprintf(tmp[0],255, "%d", i);
-#ifdef PIXMAPS
-		gtk_clist_set_pixtext(GTK_CLIST(list), i-1,0, tmp[0], 2, 
-			GNOME_PIXMAP(cd.trk[i].status==TRK_DATA?data:music)->pixmap, 
-			GNOME_PIXMAP(cd.trk[i].status==TRK_DATA?data:music)->mask);
-#endif
 	}
 
 	gtk_clist_thaw(GTK_CLIST(list));
+	for(i=0; i < 4; i++)
+		g_free(tmp[i]);
+		
 	return;
 }
 
-void dtitle_changed( GtkWidget *widget, gpointer data )
+static void dtitle_changed( GtkWidget *widget, gpointer data )
 {
 	strncpy(cd.dtitle, gtk_entry_get_text(GTK_ENTRY(widget)), DISC_INFO_LEN);
+	parse_dtitle(&cd);
 	return;
 }
 
-void activate_entry( GtkWidget *widget, GtkWidget *list )
+static void activate_entry( GtkWidget *widget, GtkWidget *list )
 {
 	int trk = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(list)));
 	strncpy(cd.trk[trk].name, 
@@ -86,7 +84,15 @@ void activate_entry( GtkWidget *widget, GtkWidget *list )
 	return;
 }
 
-void select_row_cb( GtkCList *clist,
+static void next_entry( GtkWidget *widget, GtkWidget *list )
+{
+	int trk = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(list)));
+
+	if (trk<cd.last_t)
+           gtk_clist_select_row(GTK_CLIST(list), trk, 0);
+}
+
+static void select_row_cb( GtkCList *clist,
 		    gint row,
                     gint column,
                     GdkEventButton *event,
@@ -104,11 +110,12 @@ void select_row_cb( GtkCList *clist,
 
 	return;
 }
-	
-void edit_window( void )
+
+void edit_window(GtkWidget *widget, gpointer data)
 {
 	char *titles[] = {N_("Trk"),N_("Time"),N_("Title")};
-	
+	int i;
+
 	GtkWidget *disc_entry, *disc_ext;
 	GtkWidget *label, *disc_frame, *button_box;
 	GtkWidget *main_box, *disc_table, *button;
@@ -122,7 +129,7 @@ void edit_window( void )
 
 	trwin = gtk_window_new(GTK_WINDOW_DIALOG);
 	gtk_container_border_width(GTK_CONTAINER(trwin), 5);
-	gtk_window_set_title(GTK_WINDOW(trwin), N_("Track Editor"));
+	gtk_window_set_title(GTK_WINDOW(trwin), _("Track Editor"));
 	gtk_window_set_wmclass(GTK_WINDOW(trwin), "track_editor","gtcd");
 
 	gtk_signal_connect(GTK_OBJECT(trwin), "delete_event",
@@ -130,16 +137,11 @@ void edit_window( void )
 	
 	main_box = gtk_vbox_new(FALSE, 4);
 
-#ifdef PIXMAPS
-	music = gnome_pixmap_new_from_xpm_d(music_xpm);
-	data = gnome_pixmap_new_from_xpm_d(data_xpm);
-#endif
-	
 	/* Disc area */
 	disc_table  = gtk_table_new(2, 2, FALSE);
-	disc_frame = gtk_frame_new(N_("Disc Information"));
-	label 	   = gtk_label_new(N_("Artist / Title"));
-	disc_ext   = gtk_button_new_with_label(N_("Ext Data"));
+	disc_frame = gtk_frame_new(_("Disc Information"));
+	label 	   = gtk_label_new(_("Artist / Title"));
+	disc_ext   = gtk_button_new_with_label(_("Ext Data"));
 	gtk_widget_set_sensitive(disc_ext, FALSE);
 	disc_entry = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(disc_entry), cd.dtitle);
@@ -160,10 +162,15 @@ void edit_window( void )
 	entry_box   = gtk_hbox_new(FALSE, 2);
 	track_entry = gtk_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(track_entry), cd.trk[1].name);
-	track_ext   = gtk_button_new_with_label(N_("Ext Data"));
+	track_ext   = gtk_button_new_with_label(_("Ext Data"));
 	gtk_widget_set_sensitive(track_ext, FALSE);
-	track_list  = gtk_clist_new_with_titles(3, titles);
+	track_list = gtk_clist_new(3);
+	for (i=0 ; i < 3 ; i++)
+		gtk_clist_set_column_title(GTK_CLIST(track_list), i, _(titles[i]));
+	gtk_clist_column_titles_show(GTK_CLIST(track_list));
 	gtk_clist_set_border(GTK_CLIST(track_list), GTK_SHADOW_NONE);
+	/* FIXME: Setting the column size needs to be redone in a way that
+	   supports i18n */
 	gtk_clist_set_column_width(GTK_CLIST(track_list), 0, 20);
 	gtk_clist_set_column_width(GTK_CLIST(track_list), 1, 36);
 	gtk_clist_set_policy(GTK_CLIST(track_list), GTK_POLICY_AUTOMATIC,
@@ -173,7 +180,8 @@ void edit_window( void )
 	gtk_clist_column_titles_passive(GTK_CLIST(track_list));
 	gtk_widget_set_usize(track_list, 150, 225 );
 	fill_list(track_list);
-	track_frame = gtk_frame_new(N_("Track Information"));
+
+	track_frame = gtk_frame_new(_("Track Information"));
 	
 	gtk_box_pack_start_defaults(GTK_BOX(entry_box), track_entry);
 	gtk_box_pack_start_defaults(GTK_BOX(entry_box), track_ext);
@@ -184,16 +192,12 @@ void edit_window( void )
 	/* END Track area */
 
 	button_box = gtk_hbox_new(FALSE,2);
-	button = gtk_button_new_with_label(N_("CDDB Get"));
+	/* Clear button */
+	button = gtk_button_new_with_label(_("Clear"));
 	gtk_widget_set_sensitive(button, FALSE);
 	gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
-	button = gtk_button_new_with_label(N_("Submit"));
-	gtk_widget_set_sensitive(button, FALSE);
-	gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
-	button = gtk_button_new_with_label(N_("Clear"));
-	gtk_widget_set_sensitive(button, FALSE);
-	gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
-	button = gtk_button_new_with_label(N_("Playlist"));
+	/* Playlist button */
+	button = gtk_button_new_with_label(_("Playlist"));
 	gtk_widget_set_sensitive(button, FALSE);
 	gtk_box_pack_start_defaults(GTK_BOX(button_box), button);
 	gtk_box_pack_start_defaults(GTK_BOX(main_box), button_box);
@@ -215,6 +219,8 @@ void edit_window( void )
 		GTK_SIGNAL_FUNC(dtitle_changed), NULL);
 	gtk_signal_connect(GTK_OBJECT(track_entry), "changed",
 		GTK_SIGNAL_FUNC(activate_entry), track_list);
+	gtk_signal_connect(GTK_OBJECT(track_entry), "activate",
+		GTK_SIGNAL_FUNC(next_entry), track_list);
 	
 	gtk_container_add(GTK_CONTAINER(trwin), main_box);
 	gtk_widget_show_all(trwin);
