@@ -28,8 +28,10 @@
 
 #define DATABASE_SUPPORT
 
+#include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <pwd.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -52,6 +54,9 @@
 #define FALSE (0)
 #define TRUE (!FALSE)
 
+/* make gcc happy */
+void debug( const char *fmt, ...);
+
 void debug( const char *fmt, ...)
 {
 #ifdef DEBUG
@@ -66,64 +71,41 @@ void debug( const char *fmt, ...)
 
 int tcd_init_disc( cd_struct *cd, WarnFunc msg_cb )
 {
-	char tcd_dir[60];
-	char tmp[256];
-	struct stat fs;
-	struct cdrom_volctrl vol;
-
-	debug("cdrom.c: tcd_init_disc(%p) top\n", cd );
-	tcd_opencddev( cd, msg_cb );
-
-#ifdef DATABASE_SUPPORT
-	strcpy( tcd_dir, getenv("HOME"));
-	strcat( tcd_dir, "/.tcd/" );
-	if( stat( tcd_dir, &fs ) < 0 ) 
-	{
-		if( errno == ENOENT )
-		{
-			if( mkdir(tcd_dir,S_IRWXU) )
-			{
-				snprintf( tmp, 255, "Error creating %s. Reason: %s\n", tcd_dir, strerror(errno) );
-				if( msg_cb )
-					msg_cb(tmp,"error");
-
-				debug( "cdrom.c: tcd_init_disc exiting early.\n" );
-				return -1;
-			}
-			if( msg_cb )
-				msg_cb("Created directory ~/.tcd/ for cddb files.", "info" );
-		}
-		else 
-		{
-			snprintf( tmp, 255, "Error accessing %s. Reason: %s\n", tcd_dir, strerror(errno) );
-			if( msg_cb )
-				msg_cb(tmp,"error");
-			return -1;
-		}
-	}
-#endif
+    char *homedir=NULL;
+    struct passwd *pw=NULL;
+    
+    debug("cdrom.c: tcd_init_disc(%p) top\n", cd );
+    tcd_opencddev( cd, msg_cb );
+    
+    homedir = getenv("HOME");
+    
+    if (homedir == NULL) {
+	pw = getpwuid(getuid());
 	
-	if( !cd->err ) tcd_readtoc(cd);
+	if (pw != NULL)
+	    homedir = pw->pw_dir;
+	else /* This should never happen */
+	    homedir = "/";                           
+    }
+    
+    if( !cd->err ) tcd_readtoc(cd);
 #ifdef DATABASE_SUPPORT
-	if( !cd->err ) tcd_readdiskinfo(cd);
+    if( !cd->err ) tcd_readdiskinfo(cd);
 #endif
-	if( cd->err )
-	{
-		debug( "cdrom.c: tcd_init_disc exiting early (!cd->err)\n" );
-		return(-1);
-	}
-
+    if( cd->err )
+    {
+	debug( "cdrom.c: tcd_init_disc exiting early (!cd->err)\n" );
+	return(-1);
+    }
+    
 #ifdef TCD_CHANGER_ENABLED
-	/* FIXME set capability flag here for CHANGER */
-	cd->nslots = ioctl( cd->cd_dev, CDROM_CHANGER_NSLOTS );
+    cd->nslots = ioctl( cd->cd_dev, CDROM_CHANGER_NSLOTS );
 #else
-	cd->nslots = 0;
+    cd->nslots = 0;
 #endif
-	ioctl( cd->cd_dev, CDROMVOLREAD, &vol );
-	cd->volume = vol.channel0;
-
-	debug("cdrom.c: tcd_init_disc exiting normally\n" );
-	return(0);
+    
+    debug("cdrom.c: tcd_init_disc exiting normally\n" );
+    return(0);
 }
 
 int tcd_close_disc( cd_struct *cd )
@@ -161,7 +143,6 @@ int tcd_readtoc( cd_struct *cd )
 	   My timings were all screwed up. Thanks to the cdtool
 	   source, I figured it out.
 	*/
-	cd->old_id = 0; /* Prevent errors :P */
 
     	cd->trk[C(cd->last_t+1)].toc.cdte_track = CDROM_LEADOUT;
         cd->trk[C(cd->last_t+1)].toc.cdte_format = CDROM_MSF;
@@ -191,16 +172,12 @@ int tcd_readtoc( cd_struct *cd )
 			return(-1);
                 }
 
+		cd->trk[C(i)].type = cd->trk[C(i)].toc.cdte_ctrl;
 		cd->trk[C(i)].length = cd->trk[C(i)].toc.cdte_addr.msf.minute * 60 +
                         	    cd->trk[C(i)].toc.cdte_addr.msf.second;
                 cd->trk[C(i)].start = cd->trk[C(i)].length * 75 + 
                                    cd->trk[C(i)].toc.cdte_addr.msf.frame;
 		
-		/* Must not change */
-		cd->old_id += cd->trk[C(i)].length * 
-			       (cd->trk[C(cd->last_t+1)].toc.cdte_addr.msf.minute*60+
-		                cd->trk[C(cd->last_t+1)].toc.cdte_addr.msf.second);
-
 		/* Set up the default playlist */
 		cd->playlist[i] = i;
 	}
@@ -231,11 +208,36 @@ int tcd_readtoc( cd_struct *cd )
 	return tmp;
 }
 
-/* This is some ugly code. Would be nice to modularize it */
+void tcd_recalculate(cd_struct *cd)
+{
+	int result;
+	
+	cd->cur_pos_abs = cd->sc.cdsc_absaddr.msf.minute * 60 +
+		cd->sc.cdsc_absaddr.msf.second;
+	cd->cur_frame = cd->cur_pos_abs * 75 + cd->sc.cdsc_absaddr.msf.frame;
+        
+	cd->cur_pos_rel = (cd->cur_frame - cd->trk[C(cd->cur_t)].start) / 75;
+	
+	if (cd->cur_pos_rel < 0)
+		cd->cur_pos_rel = -cd->cur_pos_rel;
+        
+	if (cd->cur_pos_rel > 0 && (result = cd->cur_pos_rel % 60) == cd->t_sec)
+		return;
+
+	cd->t_sec = result;
+	cd->t_min = cd->cur_pos_rel / 60;
+        
+	cd->cd_sec = cd->cur_pos_abs % 60;
+	cd->cd_min = cd->cur_pos_abs / 60;
+
+#ifdef TCD_CHANGER_ENABLED
+	cd->cur_disc = ioctl( cd->cd_dev, CDROM_SELECT_DISC, CDSL_CURRENT );
+#endif
+}
+
 void tcd_gettime( cd_struct *cd )
 {
 	int result;
-	struct cdrom_volctrl vol;
 
 	cd->err = FALSE;
         cd->sc.cdsc_format = CDROM_MSF;
@@ -255,49 +257,33 @@ void tcd_gettime( cd_struct *cd )
 			cd->cur_t = cd->sc.cdsc_trk;
 		else
 			cd->cur_t = 0;
-#ifndef CDROMVOLCTRL_BUG
-		vol.channel0 = cd->volume;
-		vol.channel1 = vol.channel2 = vol.channel3 = vol.channel0; 
-		ioctl( cd->cd_dev, CDROMVOLCTRL, &vol );
-		ioctl( cd->cd_dev, CDROMVOLREAD, &vol );
-		cd->volume = vol.channel0;
-#endif
-	
-		cd->cur_pos_abs = cd->sc.cdsc_absaddr.msf.minute * 60 +
-			cd->sc.cdsc_absaddr.msf.second;
-	        cd->cur_frame = cd->cur_pos_abs * 75 + cd->sc.cdsc_absaddr.msf.frame;
-        
-		cd->cur_pos_rel = (cd->cur_frame - cd->trk[C(cd->cur_t)].start) / 75;
-	
-		if (cd->cur_pos_rel < 0)
-	        	cd->cur_pos_rel = -cd->cur_pos_rel;
-        
-	  	if (cd->cur_pos_rel > 0 && (result = cd->cur_pos_rel % 60) == cd->t_sec)
-	      		return;
-
-	        cd->t_sec = result;
-	        cd->t_min = cd->cur_pos_rel / 60;
-        
-	        cd->cd_sec = cd->cur_pos_abs % 60;
-		cd->cd_min = cd->cur_pos_abs / 60;
-
-		/* Update cd->trk.status */
-		for( result = cd->first_t; result <= cd->last_t; result++ )
-		{
-			cd->trk[C(result)].status = 0;
-			if( (result == cd->cur_t) && cd->sc.cdsc_audiostatus == CDROM_AUDIO_PLAY )
-				cd->trk[C(result)].status = TRK_PLAYING;
-			if( result == cd->repeat_track )
-				cd->trk[C(result)].status = TRK_REPEAT;
-			if( cd->trk[C(result)].toc.cdte_ctrl == CDROM_DATA_TRACK )
-				cd->trk[C(result)].status = TRK_DATA;
-		}			
-#ifdef TCD_CHANGER_ENABLED
-		cd->cur_disc = ioctl( cd->cd_dev, CDROM_SELECT_DISC, CDSL_CURRENT );
-#endif
+		tcd_recalculate(cd);
 	}
 }
-	                                   
+
+int tcd_set_volume(cd_struct *cd, int volume)
+{
+        struct cdrom_volctrl vol;
+
+	vol.channel0 = volume;
+	vol.channel1 = vol.channel2 = vol.channel3 = vol.channel0;
+	
+	if(ioctl(cd->cd_dev, CDROMVOLCTRL, &vol) < 0)
+		return FALSE;
+
+	return TRUE;
+}
+
+int tcd_get_volume(cd_struct *cd)
+{
+        struct cdrom_volctrl vol;
+
+	if(ioctl(cd->cd_dev, CDROMVOLREAD, &vol) < 0)
+		return -1;
+
+	return vol.channel0;
+}	
+	                                  
 int tcd_playtracks( cd_struct *cd, int start_t, int end_t )
 {
 	struct cdrom_msf msf;
@@ -310,11 +296,9 @@ int tcd_playtracks( cd_struct *cd, int start_t, int end_t )
 	
 	ioctl( cd->cd_dev, CDROMCLOSETRAY );
 	
-	if( cd->trk[C(start_t)].status == TRK_DATA )
-	{
-		debug("cdrom.c: tcd_playtracks error. trying to play data track.\n" );
-		return -1;
-	}
+	if(cd->trk[start_t].toc.cdte_ctrl == CDROM_DATA_TRACK)
+	    start_t++;		/* FIXME bad hack */
+
 	
 	msf.cdmsf_min0 = cd->trk[start_t].toc.cdte_addr.msf.minute;
 	msf.cdmsf_sec0 = cd->trk[start_t].toc.cdte_addr.msf.second;
@@ -529,111 +513,117 @@ void tcd_opencddev( cd_struct *cd, WarnFunc msg_cb )
 	cd->isdisk=TRUE;
 }
 
+void parse_dtitle(cd_struct *cd)
+{
+  char tmp[DISC_INFO_LEN], *tmp2;
+
+  /* Parse out the individual title elements. Help from Alec M. */
+  strncpy(tmp, cd->dtitle, DISC_INFO_LEN);
+  if (tmp[0])
+     strncpy(cd->artist, strtok(tmp, "/"), DISC_INFO_LEN);
+    else
+     cd->artist[0]=0;
+
+  tmp2 = strtok(NULL, "\0");
+  if(tmp2)
+    strncpy(cd->album, tmp2+1, DISC_INFO_LEN);
+  else
+    strncpy(cd->album, "", DISC_INFO_LEN);
+}
+
 int tcd_readdiskinfo( cd_struct *cd )
 {
-	struct stat fs;
-	int i, res;
-	FILE *fp;
-	char fn[60];
-	char tmp[DISC_INFO_LEN], *tmp2;
-	char artist[DISC_INFO_LEN], album[DISC_INFO_LEN];
-	char tcd_dir[128];
-
-	if( !cd->isplayable )
-		return 0;
+    int i, res;
+    FILE *fp;
+    char fn[60];
+    char *homedir=NULL;
+    char tcd_dir[128];
+    struct passwd *pw=NULL;
+    
+    if( !cd->isplayable )
+	return 0;
+    
+    homedir = getenv("HOME");
+    
+    if (homedir == NULL) {
+	pw = getpwuid(getuid());
 	
-	strcpy( tcd_dir, getenv("HOME"));
-        strcat( tcd_dir, "/.tcd/" );
-                 
-	sprintf( fn, "%s%08lx", tcd_dir, cd->cddb_id );
-	fp = fopen(fn, "r");	
-	if( fp != NULL ) // Does the new format exist?
-	{
-		fclose(fp);
-		if( (res = tcd_readcddb( cd, fn ))<0 )
-		{
-			debug("tcd_readcddb returned an error, %d\n", res );
-			sleep(2);
-			return -1;
-		}
-		strncpy(tmp, cd->dtitle, DISC_INFO_LEN);
-		strncpy(cd->artist, strtok(tmp, "/"), DISC_INFO_LEN);
-		tmp2 = strtok(NULL, "\0");
-		if( tmp2 )
-			strncpy( cd->album, tmp2+1, DISC_INFO_LEN);
-		else
-			strncpy(cd->album, "", DISC_INFO_LEN);
-		strncpy(cd->trk[0].name, "--", TRK_NAME_LEN);
-		return 0;
-	}
-	else
-	{
+	if (pw != NULL)
+	    homedir = pw->pw_dir;
+	else /* This should never happen */
+	    homedir = "/";
+    }
+    
+    strncpy( tcd_dir, homedir, sizeof(tcd_dir) );
+    strncat( tcd_dir, "/.cddbslave/", sizeof(tcd_dir) );
+    tcd_dir[sizeof(tcd_dir) - 1] = 0;
+    
+    snprintf( fn, sizeof(fn) - 1, "%s%08lx", tcd_dir, cd->cddb_id );
 
-		sprintf( fn, "%s%ld.tcd", tcd_dir, cd->old_id );
-
-		debug("Warning, can't open new format \'%s\', trying old.\n", fn );
-		debug("opening...%s\n", fn );
-
-		if( !stat(fn, &fs) )
-		{
-			fp = fopen( fn, "r" );
-			fgets( artist, 40, fp );
-			fgets( album, 40, fp );
-			artist[strlen(artist)-1] = 0;
-			album[strlen(album)-1] = 0;
-			
-			strncpy(cd->dtitle, "", DISC_INFO_LEN);
-			strncat(cd->dtitle, artist, DISC_INFO_LEN);
-			strncat(cd->dtitle, " / ", DISC_INFO_LEN);
-			strncat(cd->dtitle, album, DISC_INFO_LEN);
-				
-			for( i = cd->first_t; i <= cd->last_t; i++ )
-			{
-				fgets( cd->trk[C(i)].name, 40, fp );
-			        cd->trk[C(i)].name[strlen(cd->trk[C(i)].name)-1] = 0;
-				cd->trk[C(i)].titled = TRUE;
-			}
-			sprintf( fn, "%s%08lx", tcd_dir, cd->cddb_id );
-			tcd_writecddb( cd, fn );
-			fclose(fp);
-			strncpy( cd->trk[0].name, "--", TRK_NAME_LEN);
-			return 0;
-		}	                                                                                       			
-		else 
-		{		
-
-			debug("Warning, can't open \'%s\' \n", fn );
-			strcpy( cd->dtitle, "Unknown / Unknown" );
-		
-			for( i = cd->first_t; i <= cd->last_t; i++ )
-			{
-				sprintf( cd->trk[C(i)].name, "Track %d", i );
-				cd->trk[C(i)].titled = FALSE;
-			}
-                        sprintf( fn, "%s%08lx", tcd_dir, cd->cddb_id );
-                        tcd_writecddb( cd, fn );
-			strcpy( cd->trk[0].name, "--" );
-			return 0;
-        	}
-	}
+    fp = fopen(fn, "r");	
+    if(fp != NULL)
+    {
 	fclose(fp);
-	return -1;
+	if((res = tcd_readcddb( cd, fn ))<0)
+	{
+	    debug("tcd_readcddb returned an error, %d\n", res );
+	    return -1;
+	}
+
+        parse_dtitle(cd);
+	strncpy(cd->trk[0].name, "--", TRK_NAME_LEN);
+	return 0;
+    }
+    else
+    {
+	/* Here's where we want to send a request to our slave */
+	tcd_call_cddb_slave(cd, "TCD", "3.0");
+	debug("Warning, can't open \'%s\' \n", fn );
+	strcpy( cd->dtitle, "Unknown / Unknown" );
+	
+	for( i = cd->first_t; i <= cd->last_t; i++ )
+	{
+	    sprintf( cd->trk[C(i)].name, "Track %d", i );
+	    cd->trk[C(i)].titled = FALSE;
+	}
+	strcpy( cd->trk[0].name, "--" );
+
+	parse_dtitle(cd);
+	strncpy(cd->trk[0].name, "--", TRK_NAME_LEN);
+
+	return 0;
+    }
+    fclose(fp);
+    return -1;
 }
 
 void tcd_writediskinfo( cd_struct *cd )
 {
-	char fn[60];
-	char home[60];
+    char fn[60];
+    char home[60];
 
-	strcpy( home, getenv("HOME"));
-	sprintf( fn, "%s/.tcd/%08lx", home, cd->cddb_id );
+    char *homedir = getenv("HOME");
+    struct passwd *pw = NULL;
+    
+    if (homedir == NULL) {
+	pw = getpwuid(getuid());
 	
-        if( tcd_writecddb(cd, fn) < 0 )
-        {
-        	perror( "tcd_writecddb" );
-        	exit(-1);
-        }
-
-	cd->needs_dbwrite=FALSE;        
-	return;
+	if (pw != NULL)
+	    homedir = pw->pw_dir;
+	else /* This should never happen */
+	    homedir = "/";
+    }
+    
+    strncpy( home, homedir, sizeof(home) );
+    home[sizeof(home) - 1] = 0;
+    snprintf( fn, sizeof(fn) - 1, "%s/.cddbslave/%08lx", home, cd->cddb_id );
+    
+    if( tcd_writecddb(cd, fn) < 0 )
+    {
+	perror( "tcd_writecddb" );
+	exit(-1);
+    }
+    
+    cd->needs_dbwrite=FALSE;        
+    return;
 }					
